import edu.bsu.cs.algorithm.*;
import edu.bsu.cs.gui.*;
import edu.bsu.cs.graph.*;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.JApplet;
import javax.swing.UIManager.*;
import javax.swing.UnsupportedLookAndFeelException;


/**	An applet that consists of a clickable button for launching the JEdit program.
	When the button area receives a mousePressed
	event, a new JeditFrame is created. This is the main class for Jedit.

	@author Aaron Nall
    @author Vellanki Kiran
    @author	Elizabeth VandenBerg
	@since 1 Jul 1998
	@version	24 May 2002
	@see	java.applet.Applet
	@see	java.awt.event.MouseListener
	@see	java.awt.event.MouseMotionListener
*/
public class JEdit extends Applet implements MouseListener, MouseMotionListener {
	Graphics 	offGraphics;
	Image 		offImage;
	FontMetrics metrics;
	Font 		font;
	String 		title = JeditFrame.baseTitle;//E. VandenBerg 2002-Jun-06
	Rectangle 	box;
	boolean 	in = false;

	/**	Called by the browser to set up the applet to receive mouse events.
	*/
	public void init() {
		font = new Font("Serif", Font.BOLD, 48);
		metrics = getFontMetrics(font);
		setBackground(Color.lightGray);
		addMouseListener(this);
		addMouseMotionListener(this);

		try { UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName() ); } //E. VandenBerg 2002-May-24
		catch (ClassNotFoundException e1) {;}	//E. VandenBerg 2002-May-24
		catch (InstantiationException e2) {;}	//E. VandenBerg 2002-May-24
		catch (IllegalAccessException e3) {;}	//E. VandenBerg 2002-May-24
		catch (UnsupportedLookAndFeelException e4) {;}//E. VandenBerg 2002-May-24

	}

        /* Other methods of the interface */

	/**	Not implemented.
	*/
	public void mouseDragged(MouseEvent e) {}

    /**	Checks whether the mouse has moved onto the button.
		@param	e	the mouse event
		@exception	oops
		@see	java.awt.event.MouseEvent
    */
	public void mouseMoved(MouseEvent e) {
		int x = e.getX(), y = e.getY();
		try {
			if  ((x > box.x) && (x < box.x + box.width) && (y > box.y) && (y < box.y + box.height))
				in = true;
			else
				in = false;
			repaint();
		} catch (Exception oops) {}
	}

	/**	If the mouse is moved off of the button, repaints the window.
		@param	e	the mouse event
		@see	java.awt.event.MouseEvent
	*/
	public void mouseExited(MouseEvent e) {
		in = false;
		repaint();
	}

	/**	Not implemented.
	*/
	public void mouseEntered(MouseEvent e) {}

	/**	Not implemented.
	*/
    public void mouseReleased(MouseEvent e) {}

	/**	Not implemented.
	*/
	public void mouseClicked(MouseEvent e) {}

	/**	If the event is on the button, creates a new JeditFrame.
		@param	e	the mouse event
		@see	java.awt.event.MouseEvent
	*/
	public void mousePressed(MouseEvent e) {
		if ((e.getX() > box.y) && (e.getY() > box.y))
			if ((e.getX() < box.x + box.width) && (e.getY() < box.y + box.height)) {
				new JeditFrame(title+" - "+JeditFrame.NEWFILENAME, this);//create a new, unnamed Jedit
				//super.setSize(0,0);//E. VandenBerg 2002-Jun-20
				//super.setVisible(false);//E. VandenBerg 2002-Jun-20
			}
			repaint(); //Clean up the display
	}

	/**	Overridden for double-buffered graphics.
		@param	g	the graphic to update
	*/
	public void update(Graphics g) {
		paint(g);
	}

	/**	Paints the button and logo in the applet.
		If the mouse is in the button area, the button will be
		painted in a "pushed-in" style.
		@param	g	the graphic to paint
	*/
	public void paint(Graphics g) {
		if (offImage == null) {
			offImage = createImage(getSize().width, getSize().height);
			offGraphics = offImage.getGraphics();
		}
		else if ((offImage.getWidth(this) != getSize().width) || (offImage.getHeight(this) != getSize().height)) {
			offImage = createImage(getSize().width, getSize().height);
			offGraphics = offImage.getGraphics();
		}

	/*	To get the text in the middle of the button        */
		offGraphics.setColor(Color.gray);
		offGraphics.fillRect(0,0,getSize().width, getSize().height);
		offGraphics.setFont(font);
		int x = (getSize().width - metrics.stringWidth(title))/2;
		//int y = (getSize().height - metrics.getHeight())/2;
		//int y = (int)((getSize().height - metrics.getHeight())/1.5);
		int y = 20 + metrics.getHeight();

        /*      Here is where the  "pushed-in" style comes into picture */
		if (! in)
			offGraphics.setColor(Color.lightGray);
		else
			offGraphics.setColor(Color.gray);
		box = new Rectangle(x-10,y-metrics.getHeight(), metrics.stringWidth(title) + 20, metrics.getHeight() + 20);
		offGraphics.fillRect(box.x, box.y, box.width, box.height);
		x -= 5;
		y -= 5;

         /*     To get the 3-D effect on the text          */
		for (int i = 0 ; i < 6; i ++) {
			if (! in)
				offGraphics.setColor(new Color(200 - 20 * i, 200 - 20 * i, 230));
			else
				offGraphics.setColor(new Color(100 - 20 * i, 100 - 20 * i, 230));
			offGraphics.drawString(title, x++, y++);
		}
		offGraphics.setColor(Color.black);
		offGraphics.drawString(title, x, y);

         /*	To get the Pushed-In effect we change the border also*/
		if (in)
			offGraphics.setColor(Color.darkGray);
		else
			offGraphics.setColor(Color.white);
		int [] xArray = {box.x - 10, box.x - 10, box.x + box.width + 10, box.x + box.width, box.x, box.x};
		int [] yArray = {box.y + box.height + 10, box.y - 10, box.y - 10, box. y, box.y, box.y + box.height};
		offGraphics.fillPolygon(xArray, yArray,6);
		int [] xArray2 = {box.x + box.width, box.x + box.width + 10, box.x + box.width + 10, box.x - 10, box.x, box.x + box.width};
		int [] yArray2 = {box.y, box.y - 10, box.y + box.height + 10, box.y + box.height + 10, box.y + box.height, box.y + box.height};
		if (in)
			offGraphics.setColor(Color.white);
		else
			offGraphics.setColor(Color.darkGray);
		offGraphics.fillPolygon(xArray2, yArray2,6);
		g.drawImage(offImage,0,0,this);
		//g.draw3DRect(box.x, box.y, box.width, box.height, true);


	}


	/**	Returns Get Info text for Jedit.
	*/
	public String getAppletInfo()
	{
		return new String(title+"Copyright (c) 1998-2006 Ball State University,  All Rights Reserved");
	}
}
