package edu.bsu.cs.gui;

import edu.bsu.cs.algorithm.*;
import edu.bsu.cs.graph.*;

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * A Frame that allows users to select an algorithm, whether or not to animate
 * the algorithm, and whether or not to run the algorithm in a new window.
 * Anyone wishing to getContentPane().add algorithms to Jedit should
 * getContentPane().add their <a href = Algorithm.java> Algorithm</a> subclass
 * after the comment <b>ADD ALGORITHMS HERE.</b>
 *
 * @author Aaron Nall
 * @author Vellanki Kiran
 * @author Sunita Upadrasta
 * @since 1 Jul 1998
 * @version 21 Apr 2001
 * @see Frame
 * @see WindowListener
 * @see MouseListener
 * @see ActionListener
 * @see Runnable
 */
public class JeditAlgorithms extends JFrame implements WindowListener,
		MouseListener, ActionListener, Runnable {

	/**
	* ****************************************************************************************************
	* ****************************************************************************************************
	* ****************************************************************************************************
	* ****************************************************************************************************
	* ****     NOTE:  PLEASE DO NOT CHANGE THE PLACE MARKS STARTING WITH \\CH			   ***************
	* ****     THEY DENOTE THE CHANGES TO BE MADE IN THE FILES TO INCLUDE A NEW ALGORITHM. ***************
	* ****************************************************************************************************
	* ****************************************************************************************************
	* ****************************************************************************************************
	* ****************************************************************************************************
	*/

	private static final long serialVersionUID = 1L;

	private JeditPanel panel;

	private JavaGraph graph;

	private boolean active = false;

	private Choice choice;

	private JButton ok, cancel;

	// private JScrollPane dsp; // Sunita Upadrasta 12 March 2006 - Commented
	// since it is not read locally.
	private JTextArea description;

	private JPanel buttonPanel, listPanel, buttonPanelParent,
			useNewFrameParent;

	private JLabel label;

	private Checkbox useNewFrame, delay;

	private JeditFrame frame;

	public Vector algorithmVector; // E. VandenBerg 2002-May-24 made public

	private int selected = 0;

	private Thread thread;

	public JeditAlgorithms(JeditFrame newFrame, boolean visible) {
		super("Algorithm Select");

		// //System.out.println("In JeditAlgorithms constructor");

		setBackground(Color.lightGray);
		algorithmVector = new Vector();

		/* ADD ALGORITHMS HERE */
		algorithmVector.addElement(new BlockFinding()); // BlockFinding created
		// by Naveen Koneru and
		// Kailash Gubbi
		algorithmVector.addElement(new BreadthSearch());
		algorithmVector.addElement(new ComponentSearch());
		algorithmVector.addElement(new DegreeSequence());
		algorithmVector.addElement(new HavelHakimi());// E. VandenBerg
		// 2003-Jan-03
		algorithmVector.addElement(new DeleteAllEdges()); // Added by Jay B.
		// April 20, 1999
		algorithmVector.addElement(new DepthSearch());
		algorithmVector.addElement(new Dijkstra1()); // Added by Jay B. May
		// 6, 1999
		algorithmVector.addElement(new Dijkstra2()); // Added by Jay B. May
		// 6, 1999
		algorithmVector.addElement(new Floyd());
		// Commented automatically to remove the algorithm from the list.
		    //algorithmVector.addElement(new PlanarityDrawing()); // Created by Adrian
		// Heinz Feb, 2001
		algorithmVector.addElement(new PlanarityTesting()); // Created by Adrian
		algorithmVector.addElement(new PlanarityDrawing());
		// Heinz
		algorithmVector.addElement(new Prim());
		algorithmVector.addElement(new PruferCode());
		algorithmVector.addElement(new PruferTree());
		algorithmVector.addElement(new GetAdjacentVertices());
        algorithmVector.addElement(new SequentialColoring());
		algorithmVector.addElement(new heuristic_coloring());
        algorithmVector.addElement(new Sequential_edge_coloring());
        //    algorithmVector.addElement(new PlanarityDrawing());
            // CH6
		// algorithmVector.addElement(new TryAlg());
		// algorithmVector.addElement(new ColorEdgesRed());
		// algorithmVector.addElement(new ColorVertices());

		// ***********************************
		// *************************************
		// ************************************* Make Change 6 above:
		// Add algorithmVector.addElement(new <yourNewAlgorithmClassName>());
		// *************************************
		// //System.out.println("Back In JeditAlgorithms constructor");

		frame = newFrame;
		panel = newFrame.getPanel();
		graph = panel.getGraph();

		// //System.out.println("Set Location");
		setLocation(frame.getX() + 10, frame.getY() + 10);// E. VandenBerg
		// 2002-Jun-18

		setSize(500, 500);
		setResizable(true);// E. VandenBerg 2002-Jun-23 changed to true
		addWindowListener(this);
		addMouseListener(this);
		getContentPane().setLayout(new GridLayout(3, 1, 20, 30));
		listPanel = new JPanel(new GridLayout(2, 1, 0, 10));
		label = new JLabel("Choose an Algorithm", JLabel.CENTER);
		label.setFont(new Font("Sans", Font.BOLD, 24));
		listPanel.add(label);

		// //System.out.println("choice popup");
		// CHOICE POPUP
		choice = new Choice();
		// //System.out.println("choice popup for loop");
		for (int i = 0; i < algorithmVector.size(); i++) {
			choice.add(((Algorithm) (algorithmVector.elementAt(i))).getName());
			// //System.out.println("adding
			// "+((Algorithm)(algorithmVector.elementAt(i))).getName());
		}
		listPanel.add(choice);
		getContentPane().add(listPanel);

		// //System.out.println("description area");
		// ALGORITHM DESCRIPTION AREA
		// dsp = new JScrollPane();//E. VandenBerg 2002-Jun-23
		description = new JTextArea();
		description.setEditable(false);
		description.setLineWrap(true);// E. VandenBerg 2002-Jun-23 changed to
		// JTextArea so can wrap
		description.setWrapStyleWord(true);// E. VandenBerg 2002-Jun-23
		// dsp.add(description);//E. VandenBerg 2002-Jun-23
		getContentPane().add(description);

		Algorithm algorithm = (Algorithm) (algorithmVector.elementAt(0));
		String temp = new String();

		/* The descriptions of each algorithm are entered */
		for (int i = 0; i < algorithm.getDescription().size(); i++) {
			temp += ((String) (algorithm.getDescription().elementAt(i)) + "\n");
			description.setText(temp);
		}
		repaint();// E. VandenBerg 2002-Jun-23

		choice.addMouseListener(this);
		ok = new JButton("OK");
		ok.setFont(new Font("Sans", Font.BOLD, 18));
		cancel = new JButton("Cancel");
		cancel.setFont(new Font("Sans", Font.BOLD, 18));
		buttonPanelParent = new JPanel(new GridLayout(2, 1, 15, 15));
		useNewFrame = new Checkbox("New Window", true);
		delay = new Checkbox("Animate", true);
		useNewFrameParent = new JPanel(new FlowLayout(FlowLayout.CENTER));
		useNewFrameParent.add(useNewFrame);
		useNewFrameParent.add(delay);
		buttonPanelParent.add(useNewFrameParent);
		buttonPanel = new JPanel(new FlowLayout(FlowLayout.CENTER, 20, 0));
		buttonPanel.add(ok);
		buttonPanel.add(cancel);
		buttonPanelParent.add(buttonPanel);
		getContentPane().add(buttonPanelParent);
		ok.addActionListener(this);
		cancel.addActionListener(this);
		setVisible(visible);
		thread = new Thread(this);
		thread.start();
		// panel.setEnable(false);

	}

	/**
	 * This constructor instantiates JeditAlgorithms with the target JeditFrame
	 * newFrame. The effects of the algorithm will be directed at the JavaGraph
	 * of newFrames's JeditPanel, or at a new JeditFrame's components, as
	 * specified by the user.
	 *
	 * @param newFrame
	 *            The target frame.
	 */
	public JeditAlgorithms(JeditFrame newFrame) {
		this(newFrame, true);
	}

	/**
	 * Responsible for making the algorithm description match the algorithm that
	 * the user has selected. Runs a separate Thread while the Frame is open.
	 */
	public void run() {
		while (true) {
			try {
				thread.sleep(500);
			} catch (Exception oops) {
			}
			if (selected == choice.getSelectedIndex())
				continue;
			description.setText("");
			Algorithm algorithm = (Algorithm) (algorithmVector.elementAt(choice
					.getSelectedIndex()));
			selected = choice.getSelectedIndex();
			String temp = new String();
			for (int i = 0; i < algorithm.getDescription().size(); i++) {
				temp += ((String) (algorithm.getDescription().elementAt(i)) + "\n");
				description.setText(temp);
			}
			// //System.out.println(temp);
			repaint();// E. VandenBerg 2002-Jun-23
		}
	}

	/**
	 * Activates.
	 *
	 * @param e
	 *            The MouseEvent.
	 */
	public void mouseEntered(MouseEvent e) {
		active = true;
	}

	/** Not implemented. */
	public void mouseExited(MouseEvent e) {
	}

	/** Not implemented. */
	public void mousePressed(MouseEvent e) {
	}

	/** Not implemented. */
	public void mouseReleased(MouseEvent e) {
	}

	/** Not implemented. */
	public void mouseClicked(MouseEvent e) {
	}

	/** Not implemented. */
	public void windowOpened(WindowEvent e) {
	}

	/** Not implemented. */
	public void windowActivated(WindowEvent e) {
	}

	/** Not implemented. */
	public void windowDeactivated(WindowEvent e) {
	}

	/**
	 * Disposes if active.
	 *
	 * @param e
	 *            The WindowEvent.
	 */
	public void windowIconified(WindowEvent e) {
		if (active)
			dispose();
	}

	/** Not implemented. */
	public void windowDeiconified(WindowEvent e) {
	}

	/**
	 * Disposes.
	 *
	 * @param e
	 *            The WindowEvent.
	 */
	public void windowClosing(WindowEvent e) {
		dispose();
	}

	/** Not implemented. */
	public void windowClosed(WindowEvent e) {
	}

	/**
	 * Called when the user clicks on one of the Frame's buttons. If the user
	 * clicks on "Cancel," the Frame is destroyed. If the user clicks on "OK,"
	 * the selected <a href = Algorithm.html> Algorithm</a> is started in a
	 * manner determined by it's type. The Algorithm will be run in the current
	 * <a href = JeditFrame.html>JeditFrame</a> or in a new Frame, depending on
	 * the user's wish. The Algorithm will be animated, if the user so desires.
	 *
	 * @param e
	 *            The ActionEvent.
	 */
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == cancel)
			dispose();
		else {

			JeditFrame nframe;
			JeditPanel npanel;
			JavaGraph ngraph;

			if (useNewFrame.getState()) {
				nframe = new JeditFrame(frame, true);
				npanel = nframe.getPanel();
				ngraph = npanel.getGraph();
				// panel.setEnable(false);
			} else {
				nframe = frame;
				npanel = frame.getPanel();
				ngraph = panel.getGraph();

			}

			// nframe.getMessagePane().append("***********************************************************\n");//E.
			// VandenBerg 2002-Jun-21
			// nframe.getMessagePane().append("Starting " +
			// ((Algorithm)(algorithmVector.elementAt(choice.getSelectedIndex()))).getName()
			// + " Algorithm.\n");//E. VandenBerg 2002-Jun-21

			npanel.setDelay(delay.getState());
			/* Launch Algorithm */
			npanel.setAlgorithm((Algorithm) (algorithmVector.elementAt(choice
					.getSelectedIndex())));
			npanel.setMouseAction(((Algorithm) (algorithmVector
					.elementAt(choice.getSelectedIndex()))).getType());
			switch (npanel.getMouseAction()) {
			case JeditPanel.ONECLICKALGORITHM:
				nframe.getMessagePane().append(
						"Click on a vertex to start the "
								+ ((Algorithm) (algorithmVector
										.elementAt(choice.getSelectedIndex())))
										.getName() + " Algorithm.\n");
				break;
			case JeditPanel.TEXTINPUTALGORITHM:
				npanel.setEnable(false);
				new InputString(npanel.getAlgorithm().getName(), npanel);
				break;
			case JeditPanel.TWOCLICKALGORITHM:
				nframe.getMessagePane().append(
						"Click on two vertices to start the "
								+ ((Algorithm) (algorithmVector
										.elementAt(choice.getSelectedIndex())))
										.getName() + " Algorithm.\n");
				break;
			case JeditPanel.AUTOMATICALGORITHM:
				Vector animationVector = new Vector();
				npanel.backup();
				npanel.getAlgorithm().callOwnConstructor(ngraph, null,
						animationVector);
				if (npanel.getAlgorithm().succeeded()) {
					Animation animate = new Animation(animationVector, npanel,
							delay.getState());
					animate.start();
				}
				npanel.repaint();
				npanel.setEnable(false);
				break;
			}
			dispose();
		}
	}
	/*
	 * public static void main (String[] args) { new JeditAlgorithms(new
	 * JeditFrame(null,false)); }
	 */
}